##############################################################################
#
# RROC-examples.r: part of the RROCcurves library
#
# RROCcurves library
#  JOSE HERNANDEZ-ORALLO, UNIVERSITAT POLITECNICA DE VALENCIA, SPAIN
#  jorallo@dsic.upv.es
#
# See "INFO.TXT" for copyright information, history, related papers, etc.
#
##############################################################################



#####################
#### DIRECTORIES ####

# Directories where the other .R files must be found and the PDF files will be output. 
# Customise this for your computer and OS.
WORKDIR <- "A:/__FAENA__/_RESEARCH/_SUBMISSIONS/2012/Regression ROC Curves/script/RROCcurves" # casa
# WORKDIR <- "E:/__FAENA__/_RESEARCH/_SUBMISSIONS/2012/Regression ROC Curves/script" # poli
setwd(WORKDIR)

source("RROC-functions.R")





######################
###### OPTIONS #######
######################

# Graphic options
NEWPLOT <- TRUE            # Draws RROC space. Use whenever you want to start a new plot. If you want to draw curves on an existing plot, set this to FALSE
NORMALISE_AXES <- FALSE    # Divide x-axis and y-axis by n (OVER/n and UNDER/n)

# PDF options
PDFOPEN <- FALSE            # If the plots are output on a PDF file
PDFCLOSE <- PDFOPEN        # Close the PDF file. This should match PDFOPEN, except when you want to draw several curves before closing.
PDFheight= 5.5             # 7 is the default, so 14 makes it double higher than wide, 5 makes letters bigger (in proportion) for just one plot
PDFwidth= 5                # (as above for width) 7 by default
pdfname <- "RROCcurve.pdf" # File name 
# mymar <- c(5, 4, 4, 2) + 0.1  # default
# mymar <- c(5, 5, 4, 2) + 0.1  # you can use this to modify the margins of your PDF file
# par(mar=mymar) 


REC_ERROR_FUNCTION <- 2              # 1 abserror, 2 squerror (Used for REC curves, not for RROC curves)
















######################
####### MAIN #########
######################


#### Initialises errorfun for REC curves (not used for RROC curves)
if (REC_ERROR_FUNCTION == 1) {
  errorfun <- abserror
} else {
  errorfun <- squerror
}









#### We define some regression models ####

# "actual" is the vector with the true values
actual <- c( 0.211, 2.725, 1.933, 3.242, 7.858, 6.061, 7.173, 3.082, 0.894, 1.203)
N <- length(actual)  # N is used for the size of the dataset

# Some regression models (predictions)
m1     <- c(-0.082, 3.323, 2.320, 1.080, 7.893, 4.983, 5.121, 3.442, 2.083, 1.112) # m1 in the paper
m2     <- c( 0.786, 2.078, 0.587, 1.676, 9.052, 5.875, 6.885, 3.038, 4.097, 0.308) # m2 in the paper (symmetric, i.e., 0 bias)
m3     <- c( 1.253, 4.232, 1.734, 5.325, 6.842, 9.325, 8.232, 3.525, 1.352, 1.778) # m3 in the paper
m4ties <- c(0.123, -0.779+2, 1.845, 4.573, 8.558, 7.392, 3.669+2, -0.422+2, 0.806, 1.245) # m4 in the paper (has ties)
m13mean <- (m1 + m3)/2          # Mean (usually better than m1 or m3 alone)
mmean  <- rep(mean(actual),10)  # The mean of the true values (just to see how it looks)
m1cal <- calibrate_regressor(m1, actual) # shifts m1 to have bias 0 (overpredictions = underpredictions)
m2cal <- calibrate_regressor(m2, actual) # shifts m2 to have bias 0 (overpredictions = underpredictions)  (although m2 was already calibrated)
m3cal <- calibrate_regressor(m3, actual) # shifts m3 to have bias 0 (overpredictions = underpredictions)









#### Some examples of REC curves

if (PDFOPEN) {
  pdf(pdfname, height= PDFheight, width= PDFwidth)
}


if (NEWPLOT) {
  xlim <- max(errorfun(m3,actual))   # I use m3 here because it is the one with maximum error on one example
  REC_space(xlim, REC_ERROR_FUNCTION)
}

PLOT_CURVE <- TRUE
RECCURVE1 <- REC_curve(m1, actual, errorfun, PLOT_CURVE, "red")
RECCURVE2 <- REC_curve(m2, actual, errorfun, PLOT_CURVE, "blue")
RECCURVE3 <- REC_curve(m3, actual, errorfun, PLOT_CURVE, "green")

RECCURVE1cal <- REC_curve(m1cal, actual, errorfun, PLOT_CURVE, "pink")
RECCURVE2cal <- REC_curve(m2cal, actual, errorfun, PLOT_CURVE, "cyan")
RECCURVE3cal <- REC_curve(m3cal, actual, errorfun, PLOT_CURVE, "yellow")

if (PDFCLOSE) {
  dev.off()
}
 









#### Now examples with RROC curves

if (PDFOPEN) {
  pdf(pdfname, height= PDFheight, width= PDFwidth)
}

#### Create the RROC plot (empty for the moment) ####

# SIZE OF THE PLOT (IN ABSOLUTE TERMS). Customise these depending on the magnitude of the errors
xlim0 <- 20
ylim0 <- 20
PLOT_DIAGONAL <- FALSE
if (NEWPLOT)
  RROC_space(NORMALISE_AXES, length(actual), xlim0, ylim0, PLOT_DIAGONAL)



#### Calculates and plots three curves ####

PLOT_POINT <- FALSE
PLOT_RECTANGLE <- FALSE
PLOT_CURVE <- TRUE

CURVE1 <- RROC_curve(m1, actual, expression("m"[1]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "red", "red", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkred" "pink"
CURVE2 <- RROC_curve(m2, actual, expression("m"[2]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "blue", "blue", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkblue" "cyan"
CURVE3 <- RROC_curve(m3, actual, expression("m"[3]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "green", "green", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # " "darkgreen"greenyellow"


#### Example of an isometric line

# alpha <- 0.8  # Touches two points
alpha <- 0.75  # Touches one point

res <- RROC_intercept(CURVE1, alpha, plot_isometric=TRUE, mycol="lightgrey", normalise_axes=NORMALISE_AXES, N) 
res

index_opt <- res[[2]]  - 1   # Minus one because the ROC curve has an extra element at the beginning
# Three ways of calculating the best shift for a given alpha (with intercept information, with curve and from scratch)
  pred <- m1
  mycurve <- CURVE1
  optimal_shift_linlin(pred, actual, index_opt = index_opt)
  optimal_shift_linlin(pred, actual, alpha, mycurve)
  optimal_shift_linlin(pred, actual, alpha)

# An old way of calculating the best shift (inefficient)
  resolution <- 1000
  optimal_shift_linlin_DEPRECATED(pred, actual, alpha, resolution)


RROC_intercept_many(list(CURVE1, CURVE2, CURVE3), alpha, plot_isometric=TRUE, mycol="lightgrey", normalise_axes=NORMALISE_AXES, N)



#### Joins all their points and calculates and plots their convex hull ####

ALLPOINTSX <- c(CURVE1[1,], CURVE2[1,], CURVE3[1,])
ALLPOINTSY <- c(CURVE1[2,], CURVE2[2,], CURVE3[2,]) 
RROC_convexhull(ALLPOINTSX, ALLPOINTSY, normalise_axes=NORMALISE_AXES, N, TRUE)  


#### Plotting more models
# Uncomment these lines for plotting these models as well
# RROC_curve(m4ties, actual, expression("m"[4]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "darkolivegreen", "darkolivegreen", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkred" "pink"
# RROC_curve(mmean, actual, NULL, PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "violet", "violet", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "purple"



#### Shows different measures on the RROC space (figure 2 of figure 12, normalised) ####
FIG2 <- FALSE
FIGNORM <- FALSE
if (FIG2 || FIGNORM) {
  PLOT_DIAGONAL <- TRUE
  if (FIG2) {
    xlim0 <- 10
    ylim0 <- 10
  }

  if (FIGNORM) {
    NORMALISE_AXES <- TRUE    # Divide x-axis and y-axis by n (OVER/n and UNDER/n)
  }

  if (NEWPLOT)
    RROC_space(NORMALISE_AXES, length(actual), xlim0, ylim0, PLOT_DIAGONAL)

  PLOT_POINT <- TRUE
  PLOT_RECTANGLE <- FALSE
  PLOT_CURVE <- FALSE
  if (FIGNORM) 
    PLOT_CURVE <- TRUE
  PLOT_BIAS <- TRUE
  PLOT_SQBIAS <- TRUE
  PLOT_USE <- TRUE
  PLOT_AE <- TRUE

  CURVE1 <- RROC_curve(m1, actual, expression("m"[1]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "red", "red", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0, PLOT_BIAS = TRUE, PLOT_SQBIAS = TRUE, PLOT_USE = TRUE, PLOT_AE = TRUE)
}







#### Optionally close the PDF file ####

if (PDFCLOSE) {
  dev.off()
}
 









 









######################################################################
### OTHER EXAMPLES 
### Basically, this code was used for some of the figures in the paper 
######################################################################




#### We define shift functions here and use them for m1. Fig. 12 in the paper ####

shiftfunction1 <- function(p, s) {
  0.9*p + 0.002*p^3 + s # Good for m1
}

shiftfunction2 <- function(p, s) {
  1.0*p - 0.004*p^3 + 1.5*s*p^2 + s 
}

# RROC_curve(m1, actual, expression("m"[1]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "purple", "purple", shiftfunction1, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkred" "pink"
# RROC_curve(m1, actual, expression("m"[1]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "orange", "orange", shiftfunction2, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkred" "pink"





#### Here we construct other models by duplicating the evidence ####

# m13 <- cbind(m1,m3) / 2  # random choice betwen m1 and m3
# actual13 <- cbind(actual, actual) / 2
# RROC_curve(m13, actual13, NULL, PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "brown4", "brown", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "orange"




#### Figures 9 and 10. Using some examples with more examples, using normal distributions ####

MYREP <- 1000
MYSD <- 10/MYREP
actualrand <- rnorm(MYREP,0,MYSD)
predrand <- rnorm(MYREP,0,MYSD)
prednoise <- actualrand + rnorm(MYREP,0,MYSD) # add noise
predmean <- rep(mean(actualrand),MYREP)

# Fig. 9
# RROC_curve(predrand, actualrand, NULL, PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "violet", "violet", NULL, normalise_axes=NORMALISE_AXES, xlim0, ylim0)  # "purple"
# RROC_curve(prednoise, actualrand, NULL, PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "orange", "orange", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "purple"
# RROC_curve(predmean, actualrand, NULL, PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "brown", "brown", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "purple"

# Fig 10
## In this order, to choose the largest first
# plot(density(predmean-actualrand), col="brown", main="")
# lines(density(prednoise-actualrand), col="orange", main="")
# lines(density(predrand-actualrand), col="violet", main="")

# A histogram could also be plotted
#### hist((predrand-actualrand))




#### This is more material used in the paper for some of the figures ####



######## FIGURE 1 ##################3

# SIZE OF THE PLOT (IN ABSOLUTE TERMS). Customise these depending on the magnitude of the errors
xlim0 <- 20
ylim0 <- 20
PLOT_DIAGONAL <- TRUE
if (NEWPLOT)
  RROC_space(NORMALISE_AXES, length(actual), xlim0, ylim0, PLOT_DIAGONAL)

PLOT_POINT <- TRUE
PLOT_RECTANGLE <- FALSE
PLOT_CURVE <- FALSE

CURVE1 <- RROC_curve(m1, actual, expression("m"[1]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "red", "red", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkred" "pink"
CURVE2 <- RROC_curve(m2, actual, expression("m"[2]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "blue", "blue", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # "darkblue" "cyan"
CURVE3 <- RROC_curve(m3, actual, expression("m"[3]), PLOT_POINT, PLOT_RECTANGLE, PLOT_CURVE, "green", "green", NULL, normalise_axes=NORMALISE_AXES, N, xlim0, ylim0)  # " "darkgreen"greenyellow"

## set NORMALISE_AXES== FALSE to get this right
## alpha= 0.8 leads to slope 0.25 and this line for models m1 and m3.
DRAW_ALPHA_ISOMETRIC <- TRUE
if (DRAW_ALPHA_ISOMETRIC) {
  intercept <- -3.82275
  slope <- 0.25
  abline(intercept, slope, col="lightgrey")
}



##### OTHER FIGURES ####


## isometric connecting m1 and m3
DRAW_m13_ISOMETRIC <- FALSE
if (DRAW_m13_ISOMETRIC) {
  intercept <- -7.133684
  slope <- 0.5674129
  abline(intercept, slope, col="lightgrey")
}

## line connecting m1 and m3
DRAW_m13_SEGMENT <- FALSE
if (DRAW_m13_SEGMENT) {
  lines(c(2.569,10.431), c(-5.676,-1.215), lty= convexlty, lwd=2) 
}


## CONVEX HULL for m1, m2 (discarded) and m3
DRAW_m13_CONVEXHULL <- FALSE
if (DRAW_m13_CONVEXHULL) {
  convexlty= "solid" # "dotted" 
  ## line connecting points m1 and m3
  lines(c(2.569,10.431), c(-5.676,-1.215), lty= convexlty, lwd=2) 
  ## vertical segment
  lines(c(2.569,2.569), c(-ylim0*2, -5.675), lty= convexlty, lwd=2) 
  lines(c(10.431,xlim0*2), c(-1.215,-1.21), lty= convexlty, lwd=2) 
}






